# -*- perl -*-

package Bundle::IspMailGate;

$VERSION = '0.01';

1;

__END__

=head1 NAME

Bundle::IspMailGate - A bundle to install the IspMailGate email filter.

=head1 SYNOPSIS

C<perl -MCPAN -e 'install IspMailGate'>

=head1 CONTENTS

MD5

MIME::Base64

Net::SMTP

Mail::Internet

MIME::Tools

IO::Tee

=head1 DESCRIPTION

This bundle includes all that's needed to run the HTML::EP package with
your WWW server.

=cut
